from .api import get, post, put, delete, head, options, patch, request
from .session import Session

class CFSpiderError(Exception):
    pass

__version__ = "1.0.1"
__all__ = [
    "get", "post", "put", "delete", "head", "options", "patch", "request",
    "Session", "CFSpiderError"
]

